/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Authors: Pablo Arroyo Loma (zioma) <zioma@linups.org>
 *           Esteban Sanchez Munoz (steve-o) <esteban@steve-o.org>
 */

/*#ifdef HAVE_CONFIG_H
#include <config.h>
#endif*/

#include <glib.h>
#include <glib/gi18n.h>

#include "buoh-comic.h"

enum {
	PROP_0,
	PROP_ID,
	PROP_URI,
	PROP_PIXBUF,
	PROP_DATE
};

struct _BuohComicPrivate {
	gchar     *id;
	gchar     *uri;
	GdkPixbuf *pixbuf;
	GDate     *date;
};

#define BUOH_COMIC_GET_PRIVATE(object) \
        (G_TYPE_INSTANCE_GET_PRIVATE ((object), BUOH_TYPE_COMIC, BuohComicPrivate))

static GObjectClass *parent_class = NULL;

static void buoh_comic_init         (BuohComic      *buoh_comic);
static void buoh_comic_class_init   (BuohComicClass *klass);
static void buoh_comic_finalize     (GObject        *object);
static void buoh_comic_get_property (GObject        *object,
				     guint           prop_id,
				     GValue         *value,
				     GParamSpec     *pspec);
static void buoh_comic_set_property (GObject        *object,
				     guint           prop_id,
				     const GValue   *value,
				     GParamSpec     *pspec);

GType
buoh_comic_get_type (void)
{
	static GType type = 0;

	if (!type) {
		static const GTypeInfo info = {
			sizeof (BuohComicClass),
			(GBaseInitFunc) NULL,
			(GBaseFinalizeFunc) NULL,
			(GClassInitFunc) buoh_comic_class_init,
			NULL,
			NULL,
			sizeof (BuohComic),
			0,
			(GInstanceInitFunc) buoh_comic_init
		};

		type = g_type_register_static (G_TYPE_OBJECT, "BuohComic",
					       &info, 0);
	}
	
	return type;
}

static void
buoh_comic_init (BuohComic *buoh_comic)
{
	   
	buoh_comic->priv = BUOH_COMIC_GET_PRIVATE (buoh_comic);

	buoh_comic->priv->id     = NULL;
	buoh_comic->priv->uri    = NULL;
	buoh_comic->priv->pixbuf = NULL;
	buoh_comic->priv->date   = NULL;
}

static void
buoh_comic_class_init (BuohComicClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);

	parent_class = g_type_class_peek_parent (klass);

	g_type_class_add_private (klass, sizeof (BuohComicPrivate));

	object_class->set_property = buoh_comic_set_property;
	object_class->get_property = buoh_comic_get_property;
	   
	g_object_class_install_property (object_class,
					 PROP_ID,
					 g_param_spec_string ("id",
							      "Id",
							      "Identificator of the comic",
							      NULL,
							      G_PARAM_READWRITE));
	g_object_class_install_property (object_class,
					 PROP_URI,
					 g_param_spec_string ("uri",
							      "URI",
							      "URI of the comic",
							      NULL,
							      G_PARAM_READWRITE));
	g_object_class_install_property (object_class,
					 PROP_PIXBUF,
					 g_param_spec_object ("pixbuf",
							      "Pixbuf",
							      "Pixbuf of the comic",
							      GDK_TYPE_PIXBUF,
							      G_PARAM_READWRITE));
	g_object_class_install_property (object_class,
					 PROP_DATE,
					 g_param_spec_pointer ("date",
							       "Date",
							       "Date release of the comic",
							       G_PARAM_READWRITE));

	object_class->finalize = buoh_comic_finalize;
}

static void
buoh_comic_finalize (GObject *object)
{
	BuohComic *comic = BUOH_COMIC (object);
	
	g_return_if_fail (BUOH_IS_COMIC (comic));

	if (comic->priv->id) {
		g_free (comic->priv->id);
		comic->priv->id = NULL;
	}

	if (comic->priv->pixbuf) {
		g_object_unref (comic->priv->pixbuf);
		comic->priv->pixbuf = NULL;
	}
	
	if (comic->priv->date) {
		g_date_free (comic->priv->date);
		comic->priv->date = NULL;
	}
	
	if (G_OBJECT_CLASS (parent_class)->finalize)
		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
}

BuohComic *
buoh_comic_new (void)
{
	BuohComic *comic;

	comic = BUOH_COMIC (g_object_new (BUOH_TYPE_COMIC, NULL));

	return comic;
}

BuohComic *
buoh_comic_new_with_info (const gchar *id, const gchar *uri,
			  const GDate *date)
{
	BuohComic *comic;

	comic = BUOH_COMIC (g_object_new (BUOH_TYPE_COMIC,
					  "id", id,
		      			  "uri", uri,
					  "date", date, NULL));
	
	return comic;
}


static void
buoh_comic_set_property (GObject      *object,
			 guint         prop_id,
			 const GValue *value,
			 GParamSpec   *pspec)
{
	BuohComic *comic = BUOH_COMIC (object);
	GDate     *date;	

	switch (prop_id) {
	case PROP_ID:
		g_free (comic->priv->id);
		comic->priv->id = g_value_dup_string (value);
		
		break;
	case PROP_URI:
		g_free (comic->priv->uri);
		comic->priv->uri = g_value_dup_string (value);
		
		break;
	case PROP_PIXBUF:
		if (comic->priv->pixbuf) {
			g_object_unref (comic->priv->pixbuf);
		}
		comic->priv->pixbuf = GDK_PIXBUF (g_value_dup_object (value));
		
		break;
	case PROP_DATE:
		if (comic->priv->date) {
			g_date_free (comic->priv->date);
		}
		date = g_value_get_pointer (value);
		
		comic->priv->date = g_date_new_dmy (g_date_get_day (date),
						    g_date_get_month (date),
						    g_date_get_year (date));
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
	}
}

static void
buoh_comic_get_property (GObject      *object,
			 guint         prop_id,
			 GValue       *value,
			 GParamSpec   *pspec)
{
	BuohComic *comic = BUOH_COMIC (object);

	switch (prop_id) {
	case PROP_ID:
		g_value_set_string (value, comic->priv->id);
		
		break;
	case PROP_URI:
		g_value_set_string (value, comic->priv->uri);
		
		break;
	case PROP_PIXBUF:
		g_value_set_object (value, comic->priv->pixbuf);
		
		break;
	case PROP_DATE:
		g_value_set_pointer (value, comic->priv->date);
		
		break;
	default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
	}
}

void
buoh_comic_set_id (BuohComic *comic, const gchar *id)
{
	g_return_if_fail (BUOH_IS_COMIC (comic));

	g_object_set (G_OBJECT (comic), "id", id, NULL);
}

void
buoh_comic_set_pixbuf (BuohComic *comic, GdkPixbuf *pixbuf)
{
	g_return_if_fail (BUOH_IS_COMIC (comic));

	g_object_set (G_OBJECT (comic), "pixbuf", pixbuf, NULL);
}

void
buoh_comic_set_date (BuohComic *comic, GDate *date)
{
	g_return_if_fail (BUOH_IS_COMIC (comic));

	g_object_set (G_OBJECT (comic), "date", date, NULL);
}

void
buoh_comic_set_pixbuf_from_file (BuohComic *comic, const gchar *filename)
{
	GdkPixbuf *pixbuf = NULL;
	
	g_return_if_fail (BUOH_IS_COMIC (comic));

	pixbuf = gdk_pixbuf_new_from_file (filename, NULL);

	if (pixbuf) {
		buoh_comic_set_pixbuf (comic, pixbuf);
		g_object_unref (pixbuf);
	}
}

gchar *
buoh_comic_get_id (BuohComic *comic)
{
	gchar *id = NULL;
	   
	g_return_val_if_fail (BUOH_IS_COMIC (comic), NULL);

	g_object_get (G_OBJECT (comic), "id", &id, NULL);

	return id;
}

gchar *
buoh_comic_get_uri (BuohComic *comic)
{
	gchar *uri = NULL;
	   
	g_return_val_if_fail (BUOH_IS_COMIC (comic), NULL);

	g_object_get (G_OBJECT (comic), "uri", &uri, NULL);

	return uri;
}

GdkPixbuf *
buoh_comic_get_pixbuf (BuohComic *comic)
{
	GdkPixbuf *pixbuf = NULL;

	g_return_val_if_fail (BUOH_IS_COMIC (comic), NULL);

	g_object_get (G_OBJECT (comic), "pixbuf", &pixbuf, NULL);

	return pixbuf;
}

GDate *
buoh_comic_get_date (BuohComic *comic)
{
	GDate *date = NULL;
	   
	g_return_val_if_fail (BUOH_IS_COMIC (comic), NULL);

	g_object_get (G_OBJECT (comic), "date", &date, NULL);

	return date;
}

GdkPixbuf *
buoh_comic_get_thumbnail (BuohComic *comic)
{
	GdkPixbuf *pixbuf = NULL;
	GdkPixbuf *thumbnail = NULL;
	gint       c_width, c_height;
	gint       d_width, d_height;

	pixbuf = comic->priv->pixbuf;

	if (pixbuf) {
		c_width = gdk_pixbuf_get_width (pixbuf);
		c_height = gdk_pixbuf_get_height (pixbuf);

		if (c_width > c_height) {
			d_width = 96;
			d_height = c_height - (((c_width - d_width) * c_height) / c_width);
		} else {
			d_height = 96;
			d_width = c_width - (((c_height - d_height) * c_width) / c_height);
		}

		thumbnail = gdk_pixbuf_scale_simple (pixbuf, d_width, d_height, GDK_INTERP_BILINEAR);

		return thumbnail;
	}

	return NULL;
}

gchar *
buoh_comic_get_filename (BuohComic *comic)
{
	gchar *filename;
	
	filename = g_path_get_basename (comic->priv->uri);
	
	return filename;
}
